function [postnm,postloc,postlocind,postTF,postcomb,T,a_r] = ...
    TFBS_DF_Bayes_MCMC_MultiTF_triangular_prior(S,PP,offset,A0,A1,A1L,priorA,T,burnin,samples, interactions,distance,weight)

% [postnm,postloc,postlocind,postTF,postcomb,T,a_r] = ...
%    TFBS_DF_Bayes_MCMC_MultiTF_triangular_prior(S,PP,offset,A0,A1,A1L,priorA,...
%    T,burnin,samples, interactions,distance,weight)

% This function implements a multiple transcription factor binding site 
% (TFBS) prediction method for a whole promoter sequence and multiple TFs.
% Estimation is implemented using a Bayesian/Markov chain Monte Carlo 
% estimation method. This function also has an option for general data 
% fusion via a positional prior. There is also option for assessing 
% combinatorial regulation.
%
% INPUT:
% S       - A row vector of size 1-by-(L+offset) consisting of integers 
%           between 1 and 4. L is the length of the sequence and offset 
%           specifies the number of base pairs that are used to extend the 
%           sequences from the beginning/left. Thus, S(offset+1:offset+L) 
%           encodes the actual promoter sequence. Base pairs are coded 
%           using integers as follows: A = 1, C = 2, G = 3, and T = 4.
% PP      - "Positional prior" specifying a (prior) probability of having a
%           binding site at each location of the sequence. PP has the same
%           size as S. PP is primarily meant to be used for general data 
%           fusion. For example, PP can be constructed from different 
%           additional data sources, such as estimated conservation 
%           probabilities (e.g. PhastCons), regulatory potential scores 
%           (e.g. ESPERR), predicted nucleosome occupancy probabilities, 
%           predicted CpG islands, etc. It is typically advantageous to 
%           properly scale/smooth the above mentioned "raw" probabilities/
%           scores before use.
% offset  - A parameter specifying the offset of the promoter start site. 
%           If offset>0, then base pairs S(1:offset) are not considered as 
%           possible locations of motifs but can be used for a higher-
%           order (Markovian) background model.
% A0      - A (4^k)-by-4 matrix specifying hyperparameter values (positive 
%           pseudo-counts) of the Dirichlet prior for the k:th order 
%           Markovian background model. A0(i,j) defines the hyperparameter 
%           for base pair j given that the previous k base pairs are i (as 
%           encoded into an integer) in the background B. An example: if
%           k=3, then i=1 corresponds to AAA, i=2 to AAC, i=3 to AAG, i=4
%           to AAT, i=5 to ACA, ..., i=64 to TTT.
% A1      - A 1-by-m cell vector specifying hyperparameter values (positive
%           pseudo-counts) of the Dirichlet priors for m different motif 
%           models (product multinomials). A1{k} is a 4-by-l(k) matrix 
%           specifying hyperparameter of the Dirichlet prior for the k:th 
%           motif. A1{k}(i,j) defines the hyperparameter of base pair i at 
%           position j in the motif k.
% A1L     - Transcription factor labels of different motif models for 
%           assessing combinatorial regulation. A1L is a 1-by-m vector of 
%           integer 1,...,p specifying a label (transcription factor) for 
%           each motif model. If A1L is left empty (or contains only a 
%           single unique number), then all motifs are assumed to belong to
%           the same TF, and combinatorial regulation equals the 
%           probability of binding.
% priorA  - Logarithm of a prior over multiple motif configurations for 
%           each fixed number of motifs. For now, this is assumed to be 
%           uniform and inverse proportional to the number of different 
%           multiple motif configurations.
% T       - An initial parse. T(1,:) defines starting locations of motifs 
%           defined by T(2,:).
% burnin  - The number of MCMC burnin iterations.
% samples - The number of MCMC samples.
% interactions - matrix of interactions of motifs 1-> these motifs interact
%                the motifs, 0->no interaction. Motifs in matrix must be in the same order than
%                in A1. Default value is no interactions (zero matrix)
% distance  - Maximum distance between interacting motifs that give
%             positive weight to motif pair. Default value is 60.
% weight      Weighting coefficient in prior for interacting motifs. This
%             weight is the maximum and it is linearly decreased to 1 so that the motif pairs thath have distances longer
%             than the maximum distance do not anymore get additional
%             weight. Default value is 4.



% OUTPUT:
% postnm  - Estimated posterior distribution of the number of motifs.
% postloc - Estimated posterior distribution over motif locations.
% postlocind - Estimated posterior distribution over motif locations (for 
%           each motif separately).
% postcomb- Estimated posterior probability of combinatorial regulation.
% T       - A final parse (from the last iteration of MCMC). T(1,:) defines
%           starting locations of motifs defined by T(2,:).
% a_r     - Acceptance-rejection rates.

% 13/11/2006 by Harri Lhdesmki
% 17/4/2008 interaction-part added by Kirsti Laurila

%===================================================
% Parameters.
%===================================================
L = length(S) - offset; % Sequence length.
bl = size(A0,1); % 4 to the background model order.
k = round(log(bl)/log(4)); % Background model order.
m = length(A1); % The number of motifs.
nTFs = length(unique(A1L)); % The number of different TFs.
l = zeros(1,m); % Motif lengths.
for i=1:m
  l(i) = size(A1{i},2);
end
lmin = min(l); % The shortest matrix.
cmax = floor(L/lmin); % The maximum # of motifs.
alpha = 0.5; % Selection probability for a motif addition.
% beta = 1 - alpha;

if isempty(offset)
  offset = k;
end
if offset<k
  error('offset needs to exceed background model order.');
end

if nargin<14
  interactions = zeros(m,m);
end

if nargin<15
  distance = 80;
end

if nargin<16
    weight=3;
end

%===================================================
% Prior over the number of motifs.
%===================================================
gamma = 0.5;
kappa = 0.5;
priorzero = 0.5;
% Prior over the number of motifs.
prior_q = [priorzero,gamma*kappa.^[0:cmax-1]];
prior_q(2:end) = (1-priorzero)*prior_q(2:end)/(sum(prior_q(2:end)));

if nTFs>1
  % Assume different TFs are independent.
  p = prior_q;
  temp = p;
  temp2 = zeros(size(p));
  
  for i=1:(nTFs-1)
    % Convolve.
    for z=0:length(prior_q)-1
      s = 0;
      s = sum(temp(1:z+1).*fliplr(p(1:z+1)));
      temp2(z+1) = s;
    end
    temp = temp2;
  end
  prior_q = temp;
  
end

prior_over_nm = log(prior_q);
prior_over_nm(2:end) = prior_over_nm(2:end) + priorA;

%temp = prior_over_nm;
%temp(2:end) = temp(2:end) - priorA;
%temp = sum(exp(temp))
%%temp = sum(exp(prior_over_nm))
%pause;



%===================================================
% Initialize output.
%===================================================
postnm = zeros(1,cmax+1);
postloc = zeros(1,L);
postlocind = zeros(m,L);
postcomb = 0;
a_r = zeros(1,burnin+samples);
accept = 0;
reject = 0;

postTF = zeros(1,nTFs);

%===================================================
% Internal variables.
%===================================================
% Sequence of k-mers.
Sd = zeros(1,L); % k-mers as decimal numbers.
b = 4.^[k-1:-1:0]';
for i=1:L
  Sd(i) = (S(offset-k+i:offset+i-1)-1)*b + 1;
end

% Start indices of motifs in internal non-cell format.
lstart = zeros(1,m);
lstop = zeros(1,m); % Corresponding stop indices.
for i=1:m
  if i==1
    lstart(1) = 1;
    lstop(1) = l(1);
  else
    lstart(i) = lstop(i-1) + 1;
    lstop(i) = lstart(i) + l(i) - 1;
  end
end

% Convert cell vector into a concatenated double array.
temp = zeros(4,lstop(m));
for i=1:m
  temp(:,lstart(i):lstop(i)) = A1{i};
end
A1 = temp;

% Initial parse.
nm = size(T,2); % # of motifs in the initial parse.
if nm==0
  Im = [];
  Mm = [];
else
  Im = T(1,:); % Starting indices of motifs.
  Mm = T(2,:); % Motif identifiers for each motif location.
end
if nm==0 % Corresponding indices for background chunks.
  Ib = [1 L];
else % Rows correspond to consequtive background chunks.
  if (Im(nm)+l(Mm(nm))-1)==L
    Ib = [[1;Im'+l(Mm)'],[Im'-1;0]];
  else
    Ib = [[1;Im'+l(Mm)'],[Im'-1;L]];
  end
end

% Data counts for background locations. NOTE: motifs counts are not used.
[N0,N1] = SequenceCounts(S,Ib,[],k,10,offset);
C0 = A0 + N0;

% For now on, work without the offset.
% Orig. sequence without offset.
S = S(offset+1:end);

% Positional prior/data fusion.
if isempty(PP)
  PP = 0.5*ones(1,L+offset); % If PP is not defined.
end
PCneg = log(1 - PP(offset+1:end));
PCpos = log(PP(offset+1:end));
if nm==0
  PCcurrent = sum(PCneg);
else
  PCcurrent = sum(PCneg);
  for i=1:nm
    PCcurrent = PCcurrent - sum(PCneg(Im(i):(Im(i)+l(Mm(i))-1))) + ...
      sum(PCpos(Im(i):(Im(i)+l(Mm(i))-1)));
  end
end

% Data counts for motif models. NOTE: to correctly compute this using code 
% below we need to remove offset first (as is done above).
N1 = zeros(4,lstop(m));
for i=1:nm % Update to correspond to initial T.
  for s=1:l(Mm(i))
    N1(S(Im(i)+s-1),lstart(Mm(i))+s-1) = ...
      N1(S(Im(i)+s-1),lstart(Mm(i))+s-1) + 1;
  end
end
C1 = A1 + N1;

Aj0 = sum(A0,2);
Aj1 = sum(A1,1);

gA0 = gammaln(A0);
gAj0 = gammaln(Aj0);
sgA0 = sum(gA0(:));
sgAj0 = sum(gAj0);

gA1 = gammaln(A1);
gAj1 = gammaln(Aj1);
sgA1 = zeros(1,m);
sgAj1 = zeros(1,m);
for i=1:m
  sgA1(i) = sum(sum(gA1(:,lstart(i):lstop(i))));
  sgAj1(i) = sum(gAj1(lstart(i):lstop(i)));
end

P0 = sgAj0 - sum(gammaln(sum(C0,2))) + sum(gammaln(C0(:))) - sgA0;
P1 = zeros(1,m);
for i=1:m
  P1(i) = sgAj1(i) - sum(gammaln(sum(C1(:,lstart(i):lstop(i)),1))) + ...
    sum(sum(gammaln(C1(:,lstart(i):lstop(i))))) - sgA1(i);
end
P = P0 + sum(P1);



%===================================================
% The main MCMC loop.
%===================================================
for r=1:(burnin+samples)
  
  %---------------------------------
  % Chooce action randomly.
  %---------------------------------
  u = rand;
  
  %---------------------------------
  % Motif addition
  %---------------------------------
  if u<alpha
    
    % Construct indices of background chunks. (NOTE: now this is
    % constructed from from scratch every time. Code could be made faster
    % by keeping this in memory.)
    if nm==0
      Ib = [1 L];
    else
      if (Im(nm)+l(Mm(nm))-1)==L
        Ib = [[1;Im'+l(Mm)'],[Im'-1;0]];
      else
        Ib = [[1;Im'+l(Mm)'],[Im'-1;L]];
      end
    end
    
    %---------------------------------------
    % Randomly chooce a proposal motif
    %---------------------------------------
    %um = unidrnd(m,[1,1]);
    um = ceil(m*rand);
    
    d = Ib(:,2) - Ib(:,1) + 1; % Length of background chunks.
    d_l = max(d - l(um) + 1,0); % Length of background chunks minus motif length.
    sum_d_l = sum(d_l); % # of all possible new motif locations.
    
    
    %------------------------------------
    % Find a possible new location.
    %------------------------------------
    if sum_d_l>0 % If a free location exists.
      
      %v = unidrnd(sum_d_l); % Chooce one free location unif. randomly.
      v = ceil(sum_d_l*rand);
      if nm==0
        loc = v;
      else
        cdf = cumsum(d_l);
        i = sum(v > cdf) + 1; % New motif will be the i:th motif (from left).
        if i==1
          loc = v;
        else
          loc = Ib(i,1) + v - cdf(i-1) - 1; % Proposed location.
        end
      end
      
      
      %----------------------------------------
      % Update counts and posterior params.
      %----------------------------------------
      for s=loc:loc+l(um)-1 % Background
        C0(Sd(s),S(s)) = C0(Sd(s),S(s)) - 1;
      end
      
      for s=1:l(um) % Motif
        C1(S(loc+s-1),lstart(um)+s-1) = C1(S(loc+s-1),lstart(um)+s-1) + 1;
      end
      
      
      %----------------------------------
      % Compute Bayes factor.
      %----------------------------------
      
      P0_new = sgAj0 - sum(gammaln(sum(C0,2))) + sum(gammaln(C0(:))) - ...
        sgA0;
      Pk_new = sgAj1(um) - ...
        sum(gammaln(sum(C1(:,lstart(um):lstop(um)),1))) + ...
        sum(sum(gammaln(C1(:,lstart(um):lstop(um))))) - sgA1(um);
      
      P_new = P - P0 + P0_new - P1(um) + Pk_new;
      
      
      % Conservation score.
      PCnew = PCcurrent;
      PCnew = PCnew - sum(PCneg(loc:(loc+l(um)-1)));
      PCnew = PCnew + sum(PCpos(loc:(loc+l(um)-1)));
      
      % Probability of the proposed move.
      B01 = log(alpha*1/sum_d_l/m);
      
      B10 = log((1-alpha)/(nm+1));
      
      % Priors.
      
      
      if nm==0
          
          Imtemp = loc;
          
          Mmtemp = um;
          
      else
          
          Imtemp = [Im(1:i-1),loc,Im(i:end)];
          
          Mmtemp = [Mm(1:i-1),um,Mm(i:end)];
          
      end
      
      Ptempold = 0;
            distance_full=max(10,distance*0.2);
      for o=1:(length(Im)-1)
          
         disttemp = Im(o+1) - Im(o) - l(Mm(o));
                     if(disttemp<distance_full)
                weighting_coefficient=weight;
           
                     else
                weighting_coefficient=-(weight-1)/(distance-distance_full)*(disttemp-distance)+1;
            end
         
         if(interactions(Mm(o),Mm(o+1))==1 && disttemp<distance)
             Ptempold = Ptempold + log(weighting_coefficient);
         end
          
      end
      
      Ptempnew = 0;
      for o=1:(length(Imtemp)-1)
          
         disttemp = Imtemp(o+1) - Imtemp(o) - l(Mmtemp(o));
                     if(disttemp<distance_full)
                weighting_coefficient=weight;
           
            else
               weighting_coefficient=-(weight-1)/(distance-distance_full)*(disttemp-distance)+1;
            end
         
         if(interactions(Mmtemp(o),Mmtemp(o+1))==1 && disttemp<distance)
             Ptempnew = Ptempnew + log(weighting_coefficient);
         end
          
      end

      
      prior_old = prior_over_nm(nm+1) + Ptempold;
      prior_new = prior_over_nm(nm+2) + Ptempnew;

         
         
          


      

      % Detailed balance condition.
      R = min([0 P_new+prior_new+B10+PCnew-P-prior_old-B01-PCcurrent]);
      %if 1 %prior_new>0 & B10>0 %exp(B_new)>0 & prior_new>0 & B10>0
      %  R = min([0 B_new+prior_new+B10-B-prior_old-B01]);
      %else
      %  R = -Inf;
      %end
      
      
      %-------------------------------
      % Accept/Reject.
      %-------------------------------
      u = log(rand);
      if u<R % Accept the proposed move.
        
        if nm==0
          Im = loc;
          Mm = um;
        else
          Im = [Im(1:i-1),loc,Im(i:end)];
          Mm = [Mm(1:i-1),um,Mm(i:end)];
        end
        nm = nm + 1;
        P = P_new;
        P0 = P0_new;
        P1(um) = Pk_new;
        PCcurrent = PCnew;
        
        accept = accept + 1;
        
      else % Reject.
        
        for s=loc:loc+l(um)-1 % Background
          C0(Sd(s),S(s)) = C0(Sd(s),S(s)) + 1;
        end
        
        for s=1:l(um) % Motif
          C1(S(loc+s-1),lstart(um)+s-1) = C1(S(loc+s-1),lstart(um)+s-1) - 1;
        end
        
        reject = reject + 1;
      end

    else
      reject = reject + 1;
    end
    
    
  %----------------------------
  % Motif Removal
  %----------------------------
  else
    
    if nm>0 % If there is a motif to remove.
      
      %i = unidrnd(nm); % Chooce one of the motifs unif. randomly.
      i = ceil(nm*rand);
      loc = Im(i);
      
      
      %----------------------------------------
      % Update counts and posterior params.
      %----------------------------------------
      for s=loc:loc+l(Mm(i))-1 % Background
        C0(Sd(s),S(s)) = C0(Sd(s),S(s)) + 1;
      end
      for s=1:l(Mm(i)) % Motif
        C1(S(loc+s-1),lstart(Mm(i))+s-1) = C1(S(loc+s-1),lstart(Mm(i))+s-1) - 1;
      end
      
      
      %----------------------------------
      % Compute Bayes factor.
      %----------------------------------
      
      P0_new = sgAj0 - sum(gammaln(sum(C0,2))) + sum(gammaln(C0(:))) - ...
        sgA0;
      Pk_new = sgAj1(Mm(i)) - ...
        sum(gammaln(sum(C1(:,lstart(Mm(i)):lstop(Mm(i))),1))) + ...
        sum(sum(gammaln(C1(:,lstart(Mm(i)):lstop(Mm(i)))))) - sgA1(Mm(i));
      
      P_new = P - P0 + P0_new - P1(Mm(i)) + Pk_new;
      
      % Conservation score.
      PCnew = PCcurrent;
      PCnew = PCnew - sum(PCpos(loc:(loc+l(Mm(i))-1)));
      PCnew = PCnew + sum(PCneg(loc:(loc+l(Mm(i))-1)));
      
      % Probability of the proposed move.
      B01 = log((1-alpha)/nm);
      
      % Probability of the reverse move. Generate a temporary index vector
      % corresponding to the proposed move.
      Imt = Im;
      Imt(i) = [];
      Mmt = Mm;
      Mmt(i) = [];
      if nm==1
        Ib = [1 L];
      else
        if (Imt(nm-1)+l(Mmt(nm-1))-1)==L
          Ib = [[1;Imt'+l(Mmt)'],[Imt'-1;0]];
        else
          Ib = [[1;Imt'+l(Mmt)'],[Imt'-1;L]];
        end
      end
      d = Ib(:,2) - Ib(:,1) + 1; % Length of background chunks.
      d_l = max(d - l(Mm(i)) + 1,0); % Length of background chunks minus motif length.
      sum_d_l = sum(d_l); % # of all possible new motif locations.
      
      B10 = log(alpha/(sum_d_l)/m);
      
      % Priors.
   
      %searching the the motifs that are next to the motif proposed to remove(in left
      %and right sides)
      
      Imtemp = Imt;
      Mmtemp = Mmt;
      
      Ptempold = 0;
      
            distance_full=max(10,distance*0.2);

         
         
          
 
      
      
      for o=1:(length(Im)-1)
          
                  
          
         disttemp = Im(o+1) - Im(o) - l(Mm(o));
         
            if(disttemp<distance_full)
                weighting_coefficient=weight;
           
            else
               weighting_coefficient=-(weight-1)/(distance-distance_full)*(disttemp-distance)+1;
            end
         
         if(interactions(Mm(o),Mm(o+1))==1 && disttemp<distance)
             Ptempold = Ptempold + log(weighting_coefficient);
         end
          
      end
      
      Ptempnew = 0;
      for o=1:(length(Imtemp)-1)
          
         disttemp = Imtemp(o+1) - Imtemp(o) - l(Mmtemp(o));
           if(disttemp<distance_full)
                weighting_coefficient=weight;
           
            else
               weighting_coefficient=-(weight-1)/(distance-distance_full)*(disttemp-distance)+1;
            end
         
         if(interactions(Mmtemp(o),Mmtemp(o+1))==1 && disttemp<distance)
             Ptempnew = Ptempnew + log(weighting_coefficient);
         end
          
      end

      
      prior_old = prior_over_nm(nm+1) + Ptempold;
      prior_new = prior_over_nm(nm) + Ptempnew;
      
      % Detailed balance condition.
      R = min([0 P_new+prior_new+B10+PCnew-P-prior_old-B01-PCcurrent]);
      %if 1 %prior_new>0 & B10>0 %exp(B_new)>0 & prior_new>0 & B10>0
      %  R = min([0 B_new+prior_new+B10-B-prior_old-B01]);
      %else
      %  R = -Inf;
      %end


      %-------------------------------
      % Accept/Reject.
      %-------------------------------
      u = log(rand);
      if u<R % Accept the proposed move
        
        P = P_new;
        P0 = P0_new;
        P1(Mm(i)) = Pk_new;
        Im(i) = [];
        Mm(i) = [];
        nm = nm - 1;
        PCcurrent = PCnew;
        
        accept = accept + 1;
        
      else % Reject.
        reject = reject + 1;
        
        for s=loc:loc+l(Mm(i))-1 % Background
          C0(Sd(s),S(s)) = C0(Sd(s),S(s)) - 1;
        end
        
        for s=1:l(Mm(i)) % Motif
          C1(S(loc+s-1),lstart(Mm(i))+s-1) = C1(S(loc+s-1),lstart(Mm(i))+s-1) + 1;
        end
        
      end

    else
      reject = reject + 1;
    end
    
  end
  
  
 %----------------------------

  % Update output.

  %----------------------------

  a_r(r) = accept/(accept+reject);

  if r>burnin

    postnm(nm+1) = postnm(nm+1) + 1;

    postloc(Im) = postloc(Im) + 1;

    indlocind = sub2ind([m,L],Mm,Im);

    postlocind(indlocind) = postlocind(indlocind) + 1;
          indTFtemp = unique(Mm); % not necessary though, I believe..
    postTF(indTFtemp) = postTF(indTFtemp) + 1;

    if nTFs>1 & length(unique(A1L(Mm)))==nTFs

      postcomb = postcomb + 1;

    end

  end

  

  %if mod(r,50000)==0

  %  disp([num2str(r),'/',num2str(burnin+samples)]);

  %end

  

end


postTF = postTF/samples;

postnm = postnm/samples;

postloc = postloc/samples;

postlocind = postlocind/samples;

if nTFs>1

  postcomb = postcomb/samples;

else % Only one TF.

  postcomb = 1 - postnm(1);

end

T = [Im;Mm];


