function [Pc,c_mu,priorA] = TFBS_DF_Likelihood(S,PP,M,B,offset)

% [Pc,c_mu,priorA] = TFBS_DF_Likelihood(S,PP,M,B,offset)
%
% This function implements a multiple transcription factor binding site 
% (TFBS) prediction method for a whole promoter sequence using a Likelihood
% based estimation method. This function also has an option for general 
% data fusion via a positional prior.
%
% INPUT:
% S       - A row vector of size 1-by-(L+offset) consisting of integers 
%           between 1 and 4. L is the length of the sequence and offset 
%           specifies the number of base pairs that are used to extend the 
%           sequences from the beginning/left. Thus, S(offset+1:offset+L) 
%           encodes the actual promoter sequence. Base pairs are coded 
%           using integers as follows: A = 1, C = 2, G = 3, and T = 4.
% PP      - "Positional prior" specifying a (prior) probability of having a
%           binding site at each location of the sequence. PP has the same
%           size as S. PP is primarily meant to be used for general data 
%           fusion. For example, PP can be constructed from different 
%           additional data sources, such as estimated conservation 
%           probabilities (e.g. PhastCons), regulatory potential scores 
%           (e.g. ESPERR), predicted nucleosome occupancy probabilities, 
%           predicted CpG islands, etc. It is typically advantageous to 
%           properly scale/smooth the above mentioned "raw" probabilities/
%           scores before use.
% M       - A 1-by-m cell vector specifying m different (normalized) motif 
%           models (product multinomials). M{k} is a 4-by-l(k) matrix 
%           specifying the k:th motif model. M{k}(i,j) defines the 
%           probability of observing a nucleotide i at position j in the 
%           motif model k.
% B       - A (4^k)-by-4 matrix specifying a k:th order Markovian 
%           background model. B(i,j) defines the probability of observing a
%           nucleotide j given that the previous k nucleotides are i (as 
%           encoded into an integer) in the background B. An example: if
%           k=3, then i=1 corresponds to AAA, i=2 to AAC, i=3 to AAG, i=4
%           to AAT, i=5 to ACA, ..., i=64 to TTT.
% offset  - A parameter specifying the offset of the promoter start site. 
%           If offset>0, then base pairs S(1:offset) are not considered as 
%           possible locations of motifs but can be used for a higher-
%           order (Markovian) background model.
%
% OUTPUT:
% Pc      - Likelihood of S containing c (c=0,1,...,floor(L/l)) motifs
%           instances.
% c_mu    - The expected number of motifs in sequence S.
% priorA  - Logarithm of the prior over multiple motif configurations for 
%           each fixed number of motifs, which is assumed to be uniform and
%           inverse proportional to the number different multiple motif 
%           configuraions.

% 08/11/2006 by Harri Lhdesmki

%---------------------------------------
% Some parameters.
%---------------------------------------
L = length(S) - offset; % Sequence length.
m = length(M); % The number of motif models.
l = zeros(1,m); % Motif lengths.
for i=1:m
  l(i) = size(M{i},2);
end
lmin = min(l); % The shortest motif model.
cmax = floor(L/min(l)); % Maximum number of non-overlapping motifs in S.
Pc = zeros(1,cmax+1); % Initialize output.
priorA = zeros(1,cmax);


%---------------------------------------
% Priors.
%---------------------------------------
if 1
  gamma = 0.5;
  %kappa = 0.25;
  kappa = 0.5; %****
  %priorzero = 0.75;
  priorzero = 0.5; %*****
  prior_q = [priorzero,gamma*kappa.^[0:cmax-1]];
  %prior_q = [1-gamma,gamma*kappa.^[0:cmax-1]];
  %prior_over_nm = [1-gamma,gamma*kappa.^[0:cmax-1],0];
  %plot(prior_over_nm(1:10),'b');
  prior_q(2:end) = (1-priorzero)*prior_q(2:end)/(sum(prior_q(2:end)));% Prior over the number of motifs.
  
%   p = prior_q;
%   temp = p;
%   temp2 = zeros(size(p));
%   
%   for i=1:(m-1)
%     % Convolve.
%     for z=0:length(prior_q)-1
%       s = 0;
%       s = sum(temp(1:z+1).*fliplr(p(1:z+1)));
%       %for k=0:z
%       %  s = s + temp(k+1)*p(z-k+1);
%       %end
%       temp2(z+1) = s;
%     end
%     temp = temp2;
%   end
%   prior_q = temp;
  
  %
  %for i=1:length(prior_q)
  %  temp2 = prior_q(1:i);
  %  for j=1:m
  %    for k=0:j
  %      temp2(k+1) = temp2(k+1) + prior_1(k+1)*prior_1(k+1)
  %    temp2 = temp2.*fliplr(prior_q(1:i));
  %  end
  %  %temp(i) = sum(prior_q(1:i).*fliplr(prior_q(1:i)));
  %  temp(i) = sum(temp2);
  %end
  %prior_q = prior_q/sum(prior_q);
  %temp(1:6)
  %sum(temp)
  %plot([0:length(prior_q)-1],prior_q,'b')
  %pause;
  
else
  gamma = 0.5;
  kappa = 0.25;
  prior_q = [1-gamma,gamma*kappa.^(0:cmax-1)];
  prior_q = prior_q/sum(prior_q);
  % Assume prior over multiple motif configurations for each fixed number of
  % motifs is uniform and inverse proportional to the number different
  % multiple motif configuraions.
end


%-------------------------------------------------
% "Scan" and store for later use.
%-------------------------------------------------
% "Scan" the additional data sources (positional prior) first.
if isempty(PP)
  PP = 0.5*ones(size(S));
end
PP = PP(offset+1:end);
BPpos = zeros(m,L);
BPneg = zeros(m,L);
for i=1:m
  for j=1:L-l(i)+1
    BPpos(i,j) = prod(PP(j:j+l(i)-1));
    BPneg(i,j) = prod(1-PP(j:j+l(i)-1));
  end
end    

% Scan using background model and combine with the positional prior.
Bm = BM_Scan(S,B,offset);
Bmm = ones(m,L);
for j=1:m
  for i=1:L-l(j)+1
    %Bmm(j,i) = prod(Bm([i:i+l(j)-1]));
    Bmm(j,i) = prod(Bm([i:i+l(j)-1]))*BPneg(j,i);
  end
end

% Scan with the motifs and combine with the positional prior.
W = zeros(m,L);
for i=1:m
  %W(i,1:L-l(i)+1) = PSWM_Scan(S,M{i},offset); % Scan using the motif model.
  W(i,1:L-l(i)+1) = PSWM_Scan(S,M{i},offset).*BPpos(i,1:L-l(i)+1);
end

% Likelihood ratios.
W = W./Bmm;

% Initialize loop-variables.
s = zeros(m,L);
%ss = zeros(1,L); % Will be initialized later.
n = zeros(m,L);
%nn = zeros(1,L); % Will be initialized later.

% Because the likelihood of the all-background model cancels out at the
% end, we ignore that completely (i.e., replace likelihood of the all
% background model with 1). This helps with numerical problems, too.
Pc(1) = 1;
%Pc(1) = prod(Bm);
%Pc(1) = sum(log(Bm));
%Pc(1) = log(prod(Bm));


%--------------------------------------------
% The single motif case.
%--------------------------------------------
for j=1:m
  for i=L-l(j)+1:-1:1
    s(j,i) = W(j,i) + s(j,i+1);
    n(j,i) = n(j,i+1) + 1;
  end
end
nn = sum(n,1);
n = zeros(m,L);

%Pc(2) = sum(s(:,1))/(L - l + 1)/m;
%Pc(2) = sum(s(:,1))/sum(L - l + 1);
Pc(2) = sum(s(:,1))/nn(1);
priorA(1) = -log(nn(1));
ss = sum(s,1);
s = zeros(m,L);


%--------------------------------------------
% The main loop.
%--------------------------------------------
for c=2:cmax % c = the number of motifs.
  
  for j=1:m
    % All possible starting positions of the first (out of c) motif.
    for i=L-l(j)-(c-1)*lmin+1:-1:1
      s(j,i) = s(j,i+1) + W(j,i)*ss(i+l(j));
      n(j,i) = n(j,i+1) + nn(i+l(j));
    end
  end
  
  nn = sum(n,1);
  n = zeros(m,L);
  
  % ss(1) is the unnormalized likelihood of finding c motifs which is
  % multiplied by the (constant) prior of different configurations of c
  % motifs in sequence S.
  Pc(c+1) = sum(s(:,1))/nn(1);
  %Pc(c+1) = sum(s(:,1))*prod((1:c)./(L - c*mean(l) + [1:c]))/(m^c);
  
  priorA(c) = -log(nn(1));
  
  ss = sum(s,1);
  s = zeros(m,L);
  
end

% Multiply by the prior over number of motifs.
Pc = Pc.*prior_q;
P_norm = sum(Pc); % Normalizing constant.
Pc = Pc/P_norm;
c_mu = sum([0:cmax].*Pc); % Expected number of motifs.
